@@ -1,6 +1,6 @@
 /* diritor.h: Iterator through entries in a directory.
  *
- * Copyright (C) 2007,2008,2010,2011,2014 Olly Betts
+ * Copyright (C) 2007,2008,2010,2011,2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "safeerrno.h"
 #include "safefcntl.h"
 #include "safesysstat.h"
+#include "safeunistd.h"
 
 #include <sys/types.h>
 
@@ -34,23 +35,34 @@
 #include <pwd.h> // For getpwuid().
 #endif
 
-#ifdef HAVE_MAGIC_H
 #include <magic.h>
-#endif
+#include <zlib.h>
 
 #include "common/noreturn.h"
 
 #include "loadfile.h"
 #include "runfilter.h" // For class ReadError.
 
+struct FileNotFound { };
+
+// Exception to signify changes should be committed, but indexing aborted.
+class CommitAndExit {
+    std::string msg;
+
+  public:
+    CommitAndExit(const char * msg_, const std::string & path, int errno_);
+    CommitAndExit(const char * msg_, int errno_);
+    CommitAndExit(const char * msg_, const char * error);
+
+    const std::string & what() const { return msg; }
+};
+
 class DirectoryIterator {
 #if defined O_NOATIME && O_NOATIME != 0
     static uid_t euid;
 #endif
 
-#ifdef HAVE_MAGIC_H
     static magic_t magic_cookie;
-#endif
 
     std::string path;
     std::string::size_type path_len;
@@ -60,6 +72,7 @@ class DirectoryIterator {
     struct stat statbuf;
     bool statbuf_valid;
     bool follow_symlinks;
+    int fd;
 
     void call_stat();
 
@@ -75,10 +88,11 @@ class DirectoryIterator {
   public:
 
     explicit DirectoryIterator(bool follow_symlinks_)
-	: dir(NULL), follow_symlinks(follow_symlinks_) { }
+	: dir(NULL), follow_symlinks(follow_symlinks_), fd(-1) { }
 
     ~DirectoryIterator() {
 	if (dir) closedir(dir);
+	if (fd >= 0) close(fd);
     }
 
     /// Start iterating through entries in @a path.
@@ -92,6 +106,10 @@ class DirectoryIterator {
     //
     //  @return false if there are no more entries.
     bool next() {
+	if (fd >= 0) {
+	    close(fd);
+	    fd = -1;
+	}
 	path.resize(path_len);
 	errno = 0;
 	do {
@@ -150,6 +168,11 @@ class DirectoryIterator {
 	return statbuf.st_mtime;
     }
 
+    time_t get_ctime() {
+	ensure_statbuf_valid();
+	return statbuf.st_ctime;
+    }
+
     const char * get_owner() {
 #ifndef __WIN32__
 	ensure_statbuf_valid();
@@ -207,18 +230,44 @@ class DirectoryIterator {
 #endif
     }
 
-#ifdef HAVE_MAGIC_H
     std::string get_magic_mimetype();
-#else
-    std::string get_magic_mimetype() { return std::string(); }
-#endif
 
     std::string file_to_string() {
 	build_path();
 	std::string out;
 	int flags = NOCACHE;
 	if (try_noatime()) flags |= NOATIME;
-	if (!load_file(path, out, flags)) throw ReadError();
+	fd = load_file_fd(path, out, flags);
+	if (fd < 0) {
+	    if (errno == ENOENT || errno == ENOTDIR) throw FileNotFound();
+	    throw ReadError("load_file failed");
+	}
+	return out;
+    }
+
+    std::string gzfile_to_string() {
+	build_path();
+	std::string out;
+	int flags = NOCACHE;
+	if (try_noatime()) flags |= NOATIME;
+	gzFile zfh = gzopen(path.c_str(), "rb");
+	if (zfh == NULL) {
+	    if (errno == ENOENT || errno == ENOTDIR) {
+		throw FileNotFound();
+	    }
+	    throw ReadError("gzopen() failed");
+	}
+	char buf[8192];
+	while (true) {
+	    int r = gzread(zfh, buf, sizeof(buf));
+	    if (r < 0) {
+		gzclose(zfh);
+		throw ReadError("gzread() failed");
+	    }
+	    out.append(buf, r);
+	    if (unsigned(r) < sizeof(buf)) break;
+	}
+	gzclose(zfh);
 	return out;
     }
 };
