@@ -1,7 +1,7 @@
 /* myhtmlparse.cc: subclass of HtmlParser for extracting text.
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2006,2007,2008,2010,2011,2012,2015 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007,2008,2010,2011,2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,9 @@
 
 #include "myhtmlparse.h"
 
+#include "datetime.h"
+#include "keyword.h"
+#include "my-html-tok.h"
 #include "stringutils.h"
 #include "utf8convert.h"
 
@@ -30,6 +33,8 @@
 
 using namespace std;
 
+static const char whitespace[] = "_ \t\r\r\f";
+
 inline void
 lowercase_string(string &str)
 {
@@ -44,7 +49,7 @@ MyHtmlParser::parse_html(const string &text, const string &charset_,
 {
     charset = charset_;
     charset_from_meta = charset_from_meta_;
-    HtmlParser::parse_html(text);
+    parse(text);
 }
 
 void
@@ -52,17 +57,18 @@ MyHtmlParser::process_text(const string &text)
 {
     if (!text.empty() && !in_script_tag && !in_style_tag) {
 	string::size_type b = text.find_first_not_of(WHITESPACE);
-	if (b) pending_space = true;
+	if (b && !pending_space) pending_space = SPACE;
 	while (b != string::npos) {
 	    if (pending_space && !target->empty())
-		*target += ' ';
+		*target += whitespace[pending_space];
 	    string::size_type e = text.find_first_of(WHITESPACE, b);
-	    pending_space = (e != string::npos);
-	    if (!pending_space) {
+	    if (e == string::npos) {
 		target->append(text.data() + b, text.size() - b);
+		pending_space = 0;
 		return;
 	    }
 	    target->append(text.data() + b, e - b);
+	    pending_space = SPACE;
 	    b = text.find_first_not_of(WHITESPACE, e + 1);
 	}
     }
@@ -71,45 +77,22 @@ MyHtmlParser::process_text(const string &text)
 bool
 MyHtmlParser::opening_tag(const string &tag)
 {
-    if (tag.empty()) return true;
-    switch (tag[0]) {
-	case 'a':
-	    if (tag == "address") pending_space = true;
+    int k = keyword(tab, tag.data(), tag.size());
+    if (k < 0)
+	return true;
+    pending_space = max(pending_space, (token_space[k] & TOKEN_SPACE_MASK));
+    switch (html_tag(k)) {
+	case P:
+	    if (pending_space < PAGE) {
+		string style;
+		if (get_parameter("style", style)) {
+		    // As produced by Libreoffice's HTML export:
+		    if (style.find("page-break-before: always") != string::npos)
+			pending_space = PAGE;
+		}
+	    }
 	    break;
-	case 'b':
-	    if (tag == "blockquote" || tag == "br") pending_space = true;
-	    break;
-	case 'c':
-	    if (tag == "center") pending_space = true;
-	    break;
-	case 'd':
-	    if (tag == "dd" || tag == "dir" || tag == "div" || tag == "dl" ||
-		tag == "dt") pending_space = true;
-	    break;
-	case 'e':
-	    if (tag == "embed") pending_space = true;
-	    break;
-	case 'f':
-	    if (tag == "fieldset" || tag == "form") pending_space = true;
-	    break;
-	case 'h':
-	    // hr, and h1, ..., h6
-	    if (tag.length() == 2 && strchr("r123456", tag[1]))
-		pending_space = true;
-	    break;
-	case 'i':
-	    if (tag == "iframe" || tag == "img" || tag == "isindex" ||
-		tag == "input") pending_space = true;
-	    break;
-	case 'k':
-	    if (tag == "keygen") pending_space = true;
-	    break;
-	case 'l':
-	    if (tag == "legend" || tag == "li" || tag == "listing")
-		pending_space = true;
-	    break;
-	case 'm':
-	    if (tag == "meta") {
+	case META: {
 		string content;
 		if (get_parameter("content", content)) {
 		    string name;
@@ -121,16 +104,33 @@ MyHtmlParser::opening_tag(const string &tag)
 				convert_to_utf8(sample, charset);
 				decode_entities(sample);
 			    }
-			} else if (name == "keywords") {
+			} else if (name == "keywords" ||
+				   name == "dcterms.subject" ||
+				   name == "dcterms.description") {
+			    // LibreOffice HTML export puts "Subject" and
+			    // "Keywords" into DCTERMS.subject, and "Comments"
+			    // into DCTERMS.description.  Best option seems to
+			    // be to treat all of these as keywords, i.e. just
+			    // more text to index, but not show in/as the
+			    // sample.
 			    if (!keywords.empty()) keywords += ' ';
 			    convert_to_utf8(content, charset);
 			    decode_entities(content);
 			    keywords += content;
-			} else if (name == "author") {
+			} else if (name == "author" ||
+				   name == "dcterms.creator" ||
+				   name == "dcterms.contributor") {
+			    // LibreOffice HTML export includes DCTERMS.creator
+			    // and DCTERMS.contributor.
 			    if (!author.empty()) author += ' ';
 			    convert_to_utf8(content, charset);
 			    decode_entities(content);
 			    author += content;
+			} else if (name == "classification") {
+			    if (!topic.empty()) topic += ' ';
+			    convert_to_utf8(content, charset);
+			    decode_entities(content);
+			    topic += content;
 			} else if (!ignoring_metarobots && name == "robots") {
 			    decode_entities(content);
 			    lowercase_string(content);
@@ -139,6 +139,10 @@ MyHtmlParser::opening_tag(const string &tag)
 				indexing_allowed = false;
 				return false;
 			    }
+			} else if (name == "created") {
+			    created = parse_datetime(content);
+			} else if (name == "dcterms.issued") {
+			    created = parse_datetime(content);
 			}
 			break;
 		    }
@@ -192,43 +196,18 @@ MyHtmlParser::opening_tag(const string &tag)
 		}
 		break;
 	    }
-	    if (tag == "marquee" || tag == "menu" || tag == "multicol")
-		pending_space = true;
+	case STYLE:
+	    in_style_tag = true;
 	    break;
-	case 'o':
-	    if (tag == "ol" || tag == "option") pending_space = true;
+	case SCRIPT:
+	    in_script_tag = true;
 	    break;
-	case 'p':
-	    if (tag == "p" || tag == "pre" || tag == "plaintext")
-		pending_space = true;
+	case TITLE:
+	    target = &title;
+	    pending_space = 0;
 	    break;
-	case 'q':
-	    if (tag == "q") pending_space = true;
-	    break;
-	case 's':
-	    if (tag == "style") {
-		in_style_tag = true;
-		break;
-	    }
-	    if (tag == "script") {
-		in_script_tag = true;
-		break;
-	    }
-	    if (tag == "select") pending_space = true;
-	    break;
-	case 't':
-	    if (tag == "table" || tag == "td" || tag == "textarea" ||
-		tag == "th") pending_space = true;
-	    else if (tag == "title") {
-		target = &title;
-		pending_space = false;
-	    }
-	    break;
-	case 'u':
-	    if (tag == "ul") pending_space = true;
-	    break;
-	case 'x':
-	    if (tag == "xmp") pending_space = true;
+	default:
+	    /* No action */
 	    break;
     }
     return true;
@@ -237,73 +216,23 @@ MyHtmlParser::opening_tag(const string &tag)
 bool
 MyHtmlParser::closing_tag(const string &tag)
 {
-    if (tag.empty()) return true;
-    switch (tag[0]) {
-	case 'a':
-	    if (tag == "address") pending_space = true;
+    int k = keyword(tab, tag.data(), tag.size());
+    if (k < 0 || (token_space[k] & NOCLOSE))
+	return true;
+    pending_space = max(pending_space, (token_space[k] & TOKEN_SPACE_MASK));
+    switch (html_tag(k)) {
+	case STYLE:
+	    in_style_tag = false;
 	    break;
-	case 'b':
-	    if (tag == "blockquote" || tag == "br") pending_space = true;
+	case SCRIPT:
+	    in_script_tag = false;
 	    break;
-	case 'c':
-	    if (tag == "center") pending_space = true;
+	case TITLE:
+	    target = &dump;
+	    pending_space = 0;
 	    break;
-	case 'd':
-	    if (tag == "dd" || tag == "dir" || tag == "div" || tag == "dl" ||
-		tag == "dt") pending_space = true;
-	    break;
-	case 'f':
-	    if (tag == "fieldset" || tag == "form") pending_space = true;
-	    break;
-	case 'h':
-	    // hr, and h1, ..., h6
-	    if (tag.length() == 2 && strchr("r123456", tag[1]))
-		pending_space = true;
-	    break;
-	case 'i':
-	    if (tag == "iframe") pending_space = true;
-	    break;
-	case 'l':
-	    if (tag == "legend" || tag == "li" || tag == "listing")
-		pending_space = true;
-	    break;
-	case 'm':
-	    if (tag == "marquee" || tag == "menu") pending_space = true;
-	    break;
-	case 'o':
-	    if (tag == "ol" || tag == "option") pending_space = true;
-	    break;
-	case 'p':
-	    if (tag == "p" || tag == "pre") pending_space = true;
-	    break;
-	case 'q':
-	    if (tag == "q") pending_space = true;
-	    break;
-	case 's':
-	    if (tag == "style") {
-		in_style_tag = false;
-		break;
-	    }
-	    if (tag == "script") {
-		in_script_tag = false;
-		break;
-	    }
-	    if (tag == "select") pending_space = true;
-	    break;
-	case 't':
-	    if (tag == "table" || tag == "td" || tag == "textarea" ||
-		tag == "th") pending_space = true;
-	    else if (tag == "title") {
-		target = &dump;
-		pending_space = false;
-		break;
-	    }
-	    break;
-	case 'u':
-	    if (tag == "ul") pending_space = true;
-	    break;
-	case 'x':
-	    if (tag == "xmp") pending_space = true;
+	default:
+	    /* No action */
 	    break;
     }
     return true;
