@@ -854,7 +854,7 @@ Fields::read_fields(Xapian::docid did) const
 	    n = names.find('\t', n);
 	    string::size_type i0 = i;
 	    i = data.find('\n', i);
-	    fields.insert(make_pair(names.substr(n0, n  - n0),
+	    fields.insert(make_pair(names.substr(n0, n - n0),
 				    data.substr(i0, i - i0)));
 	} while (++n && ++i);
     } else {
@@ -1052,7 +1052,7 @@ T(hitsperpage,	   0, 0, N, 0), // hits per page
 T(hostname,	   1, 1, N, 0), // extract hostname from URL
 T(html,		   1, 1, N, 0), // html escape string (<>&")
 T(htmlstrip,	   1, 1, N, 0), // html strip tags string (s/<[^>]*>?//g)
-T(httpheader,      2, 2, N, 0), // arbitrary HTTP header
+T(httpheader,	   2, 2, N, 0), // arbitrary HTTP header
 T(id,		   0, 0, N, 0), // docid of current doc
 T(if,		   2, 3, 1, 0), // conditional
 T(include,	   1, 1, 1, 0), // include another file
@@ -1099,7 +1099,7 @@ T(relevants,	   0, 0, N, Q), // return list of relevant documents
 T(score,	   0, 0, N, 0), // score (0-10) of current hit
 T(set,		   2, 2, N, 0), // set option value
 T(setmap,	   1, N, N, 0), // set map of option values
-T(setrelevant,     0, 1, N, Q), // set rset
+T(setrelevant,	   0, 1, N, Q), // set rset
 T(slice,	   2, 2, N, 0), // slice a list using a second list
 T(snippet,	   1, 2, N, M), // generate snippet from text
 T(split,	   1, 2, N, 0), // split a string to give a list
@@ -1438,11 +1438,11 @@ eval(const string &fmt, const vector<string> &param)
 		} else if (size < 1024) {
 		    format = "%d bytes";
 		} else {
-		    if (size < 1024*1024) {
+		    if (size < 1024 * 1024) {
 			format = "%d.%cK";
 		    } else {
 			size /= 1024;
-			if (size < 1024*1024) {
+			if (size < 1024 * 1024) {
 			    format = "%d.%cM";
 			} else {
 			    size /= 1024;
@@ -2170,12 +2170,7 @@ eval(const string &fmt, const vector<string> &param)
 
 		    if (!rset.empty()) {
 			set_expansion_scheme(*enquire, option);
-#if XAPIAN_AT_LEAST(1,3,2)
 			eset = enquire->get_eset(howmany * 2, rset, &decider);
-#else
-			eset = enquire->get_eset(howmany * 2, rset, 0,
-						 expand_param_k, &decider);
-#endif
 		    } else if (mset.size()) {
 			// invent an rset
 			Xapian::RSet tmp;
@@ -2188,12 +2183,7 @@ eval(const string &fmt, const vector<string> &param)
 			}
 
 			set_expansion_scheme(*enquire, option);
-#if XAPIAN_AT_LEAST(1,3,2)
 			eset = enquire->get_eset(howmany * 2, tmp, &decider);
-#else
-			eset = enquire->get_eset(howmany * 2, tmp, 0,
-						 expand_param_k, &decider);
-#endif
 		    }
 
 		    // Don't show more than one word with the same stem.
